/* Target definitions for aarch64 running Darwin.
   Copyright (C) 2015
   Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

GCC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

/* Enable Mach-O bits in arm code.  */
#undef TARGET_MACHO
#define TARGET_MACHO 1
#undef TARGET_ELF
#define TARGET_ELF 0

#undef  DARWIN_AARCH64
#define DARWIN_AARCH64 1

#define TARGET_64BIT 1

#define DARWIN_ARCH_SPEC "arm64"
#define DARWIN_SUBARCH_SPEC "arm64"

/* crt2.o is not used.  */
#define DARWIN_CRT2_SPEC ""

#undef SUBTARGET_EXTRA_SPECS
#define SUBTARGET_EXTRA_SPECS			\
  DARWIN_EXTRA_SPECS                            \
  { "darwin_arch", DARWIN_ARCH_SPEC },		\
  { "darwin_crt2", DARWIN_CRT2_SPEC },		\
  { "darwin_subarch", DARWIN_SUBARCH_SPEC },

#undef OS_EXTRA_SPEC_FUNCTIONS
#define OS_EXTRA_SPEC_FUNCTIONS DARWIN_EXTRA_SPEC_FUNCTIONS

/* We want -fPIC by default, unless we're using -static to compile for
   the kernel or some such.  */

#undef CC1_SPEC
#define CC1_SPEC "%(cc1_cpu) \
  %{!static:-fPIC} \
  %{g: %{!fno-eliminate-unused-debug-symbols: -feliminate-unused-debug-symbols }} " \
  DARWIN_CC1_SPEC

#define AS_NEEDS_DASH_FOR_PIPED_INPUT

#undef ASM_SPEC
#define ASM_SPEC "-c -x assembler -arch %(darwin_arch) %{static}"

#define TARGET_OS_CPP_BUILTINS()					\
  do									\
    {									\
      builtin_define ("__LITTLE_ENDIAN__");				\
      darwin_cpp_builtins (pfile);					\
      builtin_define ("__arm64");					\
      builtin_define ("__arm64__");					\
      builtin_define_with_value ("__has_feature(__X)", "0", 0);		\
    }									\
  while (0)

/* Do not use libgcc; use the existing one.  */
#undef REAL_LIBGCC_SPEC
#define REAL_LIBGCC_SPEC ""

#undef DARWIN_CRT1_SPEC
#define DARWIN_CRT1_SPEC "%{fgnu-tm: -lcrttms.o}"

#undef  LINK_GCC_C_SEQUENCE_SPEC
#define LINK_GCC_C_SEQUENCE_SPEC "-no_compact_unwind %G %{!nolibc:%L}"

/* Use dwarf-2, stabs aren't supported.  */
#undef DBX_DEBUGGING_INFO
#undef PREFERRED_DEBUGGING_TYPE
#define PREFERRED_DEBUGGING_TYPE DWARF2_DEBUG

/* Prefix in assembly output before immediate values.  */
#undef IMMEDIATE_PREFIX
#define IMMEDIATE_PREFIX "#"

/* On ARM64, the reference to the personality function in CIE is encoded
   using pcrel ARM64_RELOC_GOT relocation.  */
#undef ASM_MAYBE_OUTPUT_ENCODED_ADDR_RTX
#define ASM_MAYBE_OUTPUT_ENCODED_ADDR_RTX(FILE, ENCODING, SIZE, ADDR, DONE) \
  if ((SIZE) == 4 && ((ENCODING) & 0x70) == DW_EH_PE_pcrel)		\
    {									\
      fputs ("\t.long\t", FILE);					\
      assemble_name (FILE, XSTR (ADDR, 0));				\
      fputs ("@GOT - .", FILE);						\
      goto DONE;							\
    }

/* ARM64 Function Calling Conventions.
   In other words, long double is identical to double on iOS.  */
#undef LONG_DOUBLE_TYPE_SIZE
#define LONG_DOUBLE_TYPE_SIZE	64

/* On iOS as with other Darwin platforms, both char and wchar_t are signed
   types.  */
#undef DEFAULT_SIGNED_CHAR
#define DEFAULT_SIGNED_CHAR 1

#undef WCHAR_TYPE
#define WCHAR_TYPE "int"

/* Completly remove the TImode, so that bitsizetype is DImode.
   Because libgcc is not used and because there are no TImode operations
   in iOS, stay with DImode.
   FIXME: this may disable the use of mov pair instructions.  */
#undef  MAX_FIXED_MODE_SIZE
#define MAX_FIXED_MODE_SIZE GET_MODE_BITSIZE (DImode)
#undef  MOVE_MAX_PIECES
#define MOVE_MAX_PIECES 8

/* Avoid the use of trampolines for Ada.  */
#define USE_CUSTOM_FUNCTION_DESCRIPTORS 1

/* R18 is reserved for the platform.  */
#undef FIXED_R18
#define FIXED_R18 1

#if 0
/* On iOS, the PARM_BOUNDARY depends on the call.  For non-variadic functions,
   values consume only the space required (so the value is 8).  But for
   variadic functions, each variadic argument is assigned to the appropriate
   number of 8-byte stack slots.
   Unfortunately, GCC cannot deal with that.  */
#undef PARM_BOUNDARY
#define PARM_BOUNDARY 8
#endif

/* Define this to be nonzero if static stack checking is supported.  */
#define STACK_CHECK_STATIC_BUILTIN 1
