/* Data and functions implementing the input expansion extension services.
   Copyright (C) 2004-2014 Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3, or (at your option) any later
version.

GCC is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

#include "config.h"
#include "system.h"
#include "coretypes.h"
#include "intl.h"
#include "input.h"
#include "einput.h"

int * instance_table;

/* Same as expand_location, filling line context information in addition.  */

expanded_location
context_expand_location (source_location loc, location_lc *lcp)
{
  const struct line_map *map;
  expanded_location xloc = map_expand_location (loc, /*pmap=*/&map);

  /* If an invalid LOC reaches here somehow or we don't have an instance
     table to query, assign a constant instance id.  Otherwise, fetch the
     instance id associated with the map corresponding to the input LOC.  */

  if (map == NULL || instance_table == NULL)
    lcp->instance = 0;

  else
    {
      int map_index = ORDINARY_MAP_INDEX (linemap_check_ordinary (map),
					  line_table);

      lcp->instance = instance_table [map_index];
    }

  return xloc;
}
