/* Extensions to the basic input location expansion services, using
   extra datastructures optionally filled by language front-ends.
   Copyright (C) 2017 Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3, or (at your option) any later
version.

GCC is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

#ifndef GCC_EINPUT_H
#define GCC_EINPUT_H

#include "input.h"

/* Line context information: extra data associated with source *lines* (not
   individual source locations), filled by context_expand_location from extra
   tables that front-ends may provide by the time we reach the "final"
   pass.  */

/* For languages that have the notion of instantiating a given template
   multiple times, different linemaps can be allocated for each instance,
   which are distinguished by an instance id.  This table, which has the same
   indices as the ordinary linemaps in line_table, stores these instance
   ids.  */
extern int *instance_table;

/* The line context information per se, filled by context_expand_location
   below from data found in the tables declared above.  */

typedef struct location_lc {

  /* The instance id associated with the source line, if any.  */
  int instance;
} location_lc;

extern expanded_location context_expand_location (source_location loc,
						  location_lc *lcp);

#endif
